@@ -42,6 +42,76 @@ public:
     }
 };
 
+class Wallpaper : public KPackage::PackageStructure
+{
+public:
+    void initPackage(KPackage::Package *package)
+    {
+        package->addDirectoryDefinition("images", "images/", i18n("Images"));
+
+        QStringList mimetypes;
+        mimetypes << "image/svg" << "image/png" << "image/jpeg" << "image/jpg";
+        package->setMimeTypes("images", mimetypes);
+
+        package->setRequired("images", true);
+        package->addFileDefinition("screenshot", "screenshot.png", i18n("Screenshot"));
+        package->setAllowExternalPaths(true);
+    }
+    void pathChanged(KPackage::Package *package)
+    {
+        static bool guard = false;
+
+        if (guard) {
+            return;
+        }
+
+        guard = true;
+        QString ppath = package->path();
+        if (ppath.endsWith('/')) {
+            ppath.chop(1);
+            if (!QFile::exists(ppath)) {
+                ppath = package->path();
+            }
+        }
+
+        QFileInfo info(ppath);
+        const bool isFullPackage = info.isDir();
+        package->removeDefinition("preferred");
+        package->setRequired("images", isFullPackage);
+
+        if (isFullPackage) {
+            package->setContentsPrefixPaths(QStringList() << "contents/");
+        } else {
+            package->addFileDefinition("screenshot", info.fileName(), i18n("Preview"));
+            package->addFileDefinition("preferred", info.fileName(), QString());
+            package->setContentsPrefixPaths(QStringList());
+            package->setPath(info.path());
+        }
+
+        guard = false;
+    }
+};
+
+class SimpleContent : public KPackage::PackageStructure
+{
+public:
+    void initPackage(KPackage::Package *package)
+    {
+        package->addDirectoryDefinition("ui", "ui/", i18n("User interface"));
+    }
+    void pathChanged(KPackage::Package *package)
+    {
+        if (!package->metadata().isValid()) {
+            return;
+        }
+        if (package->metadata().serviceTypes().contains("CustomContent")) {
+            package->addFileDefinition("customcontentfile", "customcontent/CustomContentFile.qml", i18n("Custom file only for packages that contain CustomContent in their servicetypes"));
+        } else {
+            package->removeDefinition("customcontentfile");
+        }
+    }
+};
+
 void PackageStructureTest::initTestCase()
 {
     m_packagePath = QFINDTESTDATA("data/testpackage");
@@ -73,20 +143,27 @@ void PackageStructureTest::validPackages()
     QVERIFY(p.isValid());
 }
 
+void PackageStructureTest::wallpaperPackage()
+{
+    KPackage::Package p(new Wallpaper);
+    p.setPath(m_packagePath);
+    QVERIFY(p.isValid());
+
+    KPackage::Package p2(new Wallpaper);
+    p2.setPath(m_packagePath + "/contents/images/empty.png");
+    QVERIFY(p2.isValid());
+}
+
 void PackageStructureTest::copyPerformance()
 {
     // seed the cache first
     ps.filePath("mainscript");
 
-    QTime t;
-    t.start();
-
-    for (int i = 0; i < 100000; ++i) {
+    QBENCHMARK {
         KPackage::Package foo(ps);
         const QString bar = foo.filePath("mainscript");
+        Q_UNUSED(bar);
     }
-
-    QVERIFY(t.elapsed() < 400);
 }
 
 void PackageStructureTest::mutateAfterCopy()
@@ -269,5 +346,22 @@ void PackageStructureTest::mimeTypes()
     QCOMPARE(ps.mimeTypes("theme"), mimeTypes);
 }
 
+void PackageStructureTest::customContent()
+{
+    KPackage::Package p(new SimpleContent);
+    p.setPath(QFINDTESTDATA("data/simplecontent"));
+    QVERIFY(p.isValid());
+    QCOMPARE(p.filePath("customcontentfile"), QString());
+
+    p.setPath(QFINDTESTDATA("data/customcontent"));
+    const QString expected = QFINDTESTDATA("data/customcontent") + "/contents/customcontent/CustomContentFile.qml";
+    QCOMPARE(p.filePath("customcontentfile"), expected);
+    QVERIFY(p.isValid());
+
+    p.setPath(QFINDTESTDATA("data/simplecontent"));
+    QVERIFY(p.isValid());
+    QCOMPARE(p.filePath("customcontentfile"), QString());
+}
+
 QTEST_MAIN(PackageStructureTest)
 
